#ifndef __TaylorsData_Article_h__
#define __TaylorsData_Article_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <TaylorsData/Category.h>
#include <TaylorsData/Status.h>
#include <TaylorsData/Location.h>
#include <TaylorsData/SubLocation.h>
#include <TaylorsData/Image.h>
#include <TaylorsData/ArticleMetaItem.h>
#include <TaylorsData/Variant.h>
#include <TaylorsData/Units.h>

#include <string>
#include <vector>
#include <map>
#include <set>

namespace TaylorsData
{
	class ArticleMap;
	class ArticlePtrMap;

	class TaylorsData_Export Article : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					int _nArticleId;

				public:
					Key();
					Key( int nArticleId );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int articleId() const;

					static Key fromRow( const RfcDal::DalDatabase::Row& row );
			};

			static const Key keyInvalid;
			static const Key keyDefault;

			class TaylorsData_Export KeySet : public std::set<Key>
			{
				public:
					KeySet();
					KeySet( const KeySet& keyS );
					KeySet( const ArticleMap& articleS );
					KeySet( const ArticlePtrMap& articleS );

					KeySet add( const KeySet& keyS ) const;
					KeySet remove( const KeySet& keyS ) const;

					static KeySet fromResult( const RfcDal::DalDatabase::Result& result );
			};

			class TaylorsData_Export CategoryChangeMap : public std::map<Category::Key, Change>
			{
			};

			class TaylorsData_Export ImageChangeMap : public std::map<Image::Key, Change>
			{
			};

			class TaylorsData_Export ArticleMetaItemChangeMap : public std::map<ArticleMetaItem::Key, Change>
			{
			};

			enum Shape{ shapeSquare, shapeCube, shapeCircle, shapeCylinder };

		private:
			Key _key;
			const Category* _pcategory;
			CategoryPtrMap _categoryS;
			CategoryChangeMap _categoryChangeS;
			std::string _strShortDesc;
			std::string _strDescription;
			std::string _strFaults;
			std::string _strCirca;
			double _dPurchasePrice;
			double _dPrice;
			double _dReducedPrice;
			int _nStock;
			int _nDimH;
			int _nDimW;
			int _nDimD;
			std::string _strPhoto0;
			int _nDimX0;
			int _nDimY0;
			std::string _strPhoto1;
			int _nDimX1;
			int _nDimY1;
			std::string _strPhoto2;
			int _nDimX2;
			int _nDimY2;
			std::string _strPhoto3;
			int _nDimX3;
			int _nDimY3;
			const Status* _pstatus;
			const Location* _plocation;
			const SubLocation* _psubLocation;
			std::string _strInventoryNo;
			bool _bIsExempt1;
			bool _bIsExempt2;
			bool _bIsExempt3;
			bool _bIsExempt4;
			std::string _strNote;
			std::string _strRLID;
			int _nCADAID;
			std::string _strUpdated;
			ImageMap _imageS;
			ImageChangeMap  _imageChangeS;
			ArticleMetaItemMap _metaItemS;
			ArticleMetaItemChangeMap _metaItemChangeS;

		public:
			Article();
			Article( const Key& key );
			Article( const Article& article );
			virtual ~Article();

			const Article& operator=( const Article& article );

			const Key& key() const;
			const Category& category() const;
			const CategoryPtrMap& categorys() const;
			const CategoryChangeMap& categoryChanges();
			const std::string& shortDesc() const;
			const std::string& description() const;
			const std::string& faults() const;
			const std::string& circa() const;
			double purchasePrice() const;
			double price() const;
			double reducedPrice() const;
			int stock() const;
			int dimH() const;
			int dimW() const;
			int dimD() const;
			const std::string& photo0() const;
			int dimX0() const;
			int dimY0() const;
			const std::string& photo1() const;
			int dimX1() const;
			int dimY1() const;
			const std::string& photo2() const;
			int dimX2() const;
			int dimY2() const;
			const std::string& photo3() const;
			int dimX3() const;
			int dimY3() const;
			const Status& status() const;
			const Location& location() const;
			const SubLocation& subLocation() const;
			const std::string& inventoryNo() const;
			bool isExempt1() const;
			bool isExempt2() const;
			bool isExempt3() const;
			bool isExempt4() const;
			const std::string& note() const;
			const std::string& rlid() const;
			int cadaid() const;
			const std::string& updated() const;
			const ImageMap& images() const;
			const ArticleMetaItemMap metaItems() const;

			bool hasCategory() const;
			bool hasStatus() const;
			bool hasLocation() const;
			bool hasSubLocation() const;

			Variant macro( const std::string& strMacro );
			std::string format( const std::string& strFormat, bool bTitle /*= false*/ );

			void setCategory( const Category& category );
			void addCategory( const Category& category );
			void delCategory( const Category& category );
			void setShortDesc( const std::string& strShortDesc );
			void setDescription( const std::string& strDescription );
			void setFaults( const std::string& strFaults );
			void setCirca( const std::string& strCirca );
			void setPurchasePrice( double dPurchasePrice );
			void setPrice( double dPrice );
			void setReducedPrice( double dReducedPrice );
			void setStock( int nStock );
			void setDimH( int nDimH );
			void setDimW( int nDimW );
			void setDimD( int nDimD );
			void setPhoto0( const std::string& strPhoto0 );
			void setDimX0( int nDimX0 );
			void setDimY0( int nDimY0 );
			void setPhoto1( const std::string& strPhoto1 );
			void setDimX1( int nDimX1 );
			void setDimY1( int nDimY1 );
			void setPhoto2( const std::string& strPhoto2 );
			void setDimX2( int nDimX2 );
			void setDimY2( int nDimY2 );
			void setPhoto3( const std::string& strPhoto3 );
			void setDimX3( int nDimX3 );
			void setDimY3( int nDimY3 );
			void setStatus( const Status& status );
			void setLocation( const Location& location );
			void setSubLocation( const SubLocation& subLocation );
			void setInventoryNo( const std::string& strInventoryNo );
			void setExempt1( bool bIsExempt1 );
			void setExempt2( bool bIsExempt2 );
			void setExempt3( bool bIsExempt3 );
			void setExempt4( bool bIsExempt4 );
			void setNote( const std::string& strNote );
			void setRLID( const std::string& strRLID );
			void setCADAID( int nCADAID );
			ImageMap& images();
			Image& addImage( const Image& image );
			void modifyImage(const Image& image );
			void delImage( const Image& image );
			ArticleMetaItemMap& metaItems();
			ArticleMetaItem& addMetaItem( const ArticleMetaItem& item );
			ArticleMetaItem& modifyMetaItem( const ArticleMetaItem& item );
			void delMetaItem( const ArticleMetaItem& item );

			static std::string quarters( double d, Units::Unit unitDisplay );
			std::string formatDimensions( Units::Unit unitDisplay, Shape* pnShape = NULL ) const;
			static std::string formatDimensions( double dHeight, double dWidth, double dDepth,
														Units::Unit unitStore, Units::Unit unitDisplay,
														Shape* pnShape = NULL );
			static const std::vector<std::string>& units();
			static std::string inventoryNo( const std::string& strContainer );

			void fromRow( RfcDal::DalDatabase& db, const RfcDal::DalDatabase::Row& row );
			static Article fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Article create( Article& article, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

		protected:
			CategoryPtrMap loadCategorys( RfcDal::DalDatabase& conn ) const;
			ImageMap loadImages( RfcDal::DalDatabase& conn ) const;
			ArticleMetaItemMap loadMetaItems( RfcDal::DalDatabase& conn ) const;

			static void createCategorys( Article& article, int nArticleIdNew, RfcDal::DalDatabase& conn );
			static void createImages( Article& article, int nArticleIdNew, RfcDal::DalDatabase& conn );
			static void createMetaItems( Article& article, int nArticleIdNew, RfcDal::DalDatabase& conn );

			void updateCategorys( RfcDal::DalDatabase& conn );
			void updateImages( RfcDal::DalDatabase& conn );
			void updateMetaItems( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export ArticlePtrMap : public std::map<Article::Key, Article*>
	{
	};

	class TaylorsData_Export ArticleMap : public std::map<Article::Key, Article>
	{
		public:
			static ArticleMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
			ArticlePtrMap merge( const ArticleMap& articleS );
	};
};

#endif /* !__TaylorsData_Article_h__ */
