#ifndef __TaylorsData_BulkPurchase_h__
#define __TaylorsData_BulkPurchase_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <TaylorsData/BulkPurchaseType.h>
#include <TaylorsData/Article.h>

#include <Base4c/Date.h>

namespace TaylorsData
{
	class Container;

	class TaylorsData_Export BulkPurchase : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					const Container* _pcontainer;
					Base4c::Date _datePurchase;
					std::string _strSource;

				public:
					Key();
					Key( const Container& container, const Base4c::Date& datePurchase, const std::string& strSource );
					Key( const Container& container );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Container& container() const;
					const Base4c::Date& purchaseDate() const;
					const std::string& source() const;
			};

		private:
			Key _key;
			const BulkPurchaseType* _ptype;
			double _dXRate;
			double _dPRate;
			double _dTRate;

		public:
			BulkPurchase();
			BulkPurchase( const Key& key );
			BulkPurchase( const BulkPurchase& purchase );
			virtual ~BulkPurchase();

			BulkPurchase& operator=( const BulkPurchase& purchase );

			const Key& key() const;
			const BulkPurchaseType& bulkPurchaseType() const;
			double xRate() const;
			double pRate() const;
			double tRate() const;

			ArticlePtrMap articles() const;

			void setBulkPurchaseType( const BulkPurchaseType& type );
			void setXRate( double dXRate );
			void setPRate( double dPRate );
			void setTRate( double dTRate );

			static BulkPurchase fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static BulkPurchase create( const BulkPurchase& purchase, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export BulkPurchaseMap : public std::map<BulkPurchase::Key, BulkPurchase>
	{
		public:
			static BulkPurchaseMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_BulkPurchase_h__ */
