#ifndef __TaylorsData_BulkPurchaseType_h__
#define __TaylorsData_BulkPurchaseType_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>
#include <TaylorsData/MetaItem.h>

#include <string>
#include <map>
#include <set>

namespace TaylorsData
{
	class MetaItem;

	class TaylorsData_Export BulkPurchaseType : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strBulkPurchaseType;

				public:
					Key();
					Key( const std::string& strBulkPurchaseType );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& bulkPurchaseType() const;
			};

			class TaylorsData_Export Column
			{
				public:
					class TaylorsData_Export Key
					{
						private:
							int _nColumn;

						public:
							Key();
							Key( int nColumn );
							Key( const Key& key );
							virtual ~Key();

							const Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							int column() const;
					};

				private:
					Key _key;
					std::string _strSource;
					std::string _strMethod;
					MetaItem* _pmeta;

				public:
					Column();
					Column( const Key& key );
					Column( const Column& column );
					virtual ~Column();

					const Column& operator=( const Column& column );

					const Key& key() const;
					const std::string& source() const;
					const std::string& method() const;
					const MetaItem* metaItem() const;

					void setSource( const std::string& strSource );
					void setMethod( const std::string& strMethod );

					static Column fromFormat( const std::string& strFormat );
			};

			class TaylorsData_Export ColumnMap : public std::map<Column::Key, Column>
			{
				public:
					static ColumnMap fromFormat( const std::string& strFormat );
			};

		private:
			Key _key;
			std::string _strDescription;
			std::string _strFormat;
			const MetaItem* _psource;
			ColumnMap _columnS;

		public:
			BulkPurchaseType();
			BulkPurchaseType( const Key& key );
			BulkPurchaseType( const BulkPurchaseType& type );
			virtual ~BulkPurchaseType();

			const BulkPurchaseType& operator=( const BulkPurchaseType& type );

			const Key& key() const;
			const std::string& description() const;
			const std::string& format() const;
			const MetaItem& source() const;
			const ColumnMap& columns() const;

			std::set<std::string> sources() const;

			void setDescription( const std::string& strDescription );
			void setFormat( const std::string& strFormat );
			void setSource( const MetaItem& source );

			static BulkPurchaseType fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static BulkPurchaseType create( const BulkPurchaseType& type, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export BulkPurchaseTypeMap : public std::map<BulkPurchaseType::Key, BulkPurchaseType>
	{
		public:
			static BulkPurchaseTypeMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_BulkPurchaseType_h__ */
