#ifndef __TaylorsData_Category_h__
#define __TaylorsData_Category_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <TaylorsData/Site.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class CategoryPtrMap;

	class TaylorsData_Export Category : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strCategory;

				public:
					Key();
					Key( const std::string& strCategory );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& category() const;
			};

		private:
			Key _key;
			std::string _strParent;
			bool _bIsSuper;
			const Site* _psite;
			std::string _strDescription;
			std::string _strPhoto0;
			int _nDimX0;
			int _nDimY0;
			int _nSeq;
			std::string _strAbsoluteURI;
			CategoryPtrMap* _pchildren;

		public:
			Category();
			Category( const Key& key );
			Category( const Category& category );
			virtual ~Category();

			const Category& operator=( const Category& category );

			const Key& key() const;
			bool hasParent() const;
			Category& parent() const;
			bool isSuper() const;
			const Site& site() const;
			const std::string& description() const;
			const std::string& photo0() const;
			int dimX0() const;
			int dimY0() const;
			int seq() const;
			const std::string& absoluteURI() const;
			const CategoryPtrMap& children() const;

			Category& parent();
			void setParent( const Category& catParent );
			void setSuper( bool bIsSuper );
			void setSite( const Site& site );
			void setDescription( const std::string& strDescription );
			void setPhoto0( const std::string& strPhoto0 );
			void setDimX0( int nDimX0 );
			void setDimY0( int nDimY0 );
			void setSeq( int nSeq );
			void setAbsoluteURI( const std::string& strAbsoluteURI );
			CategoryPtrMap& children();

			static Category fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Category create( const Category& category, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class CategoryMap : public std::map<Category::Key, Category>
	{
		public:
			static CategoryMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};

	class CategoryPtrMap : public std::map<Category::Key, Category*>
	{
		public:
			static CategoryPtrMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
};

#endif /* !__TaylorsData_Category_h__ */
