#ifndef __TaylorsData_Container_h__
#define __TaylorsData_Container_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>
#include <TaylorsData/BulkPurchase.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Container : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strContainer;

				public:
					Key();
					Key( const std::string& strContainer );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& container() const;
			};

		private:
			Key _key;
			int _nYear;
			BulkPurchaseMap _purchaseS;
//			BulkPurchase _purchase;

		public:
			Container();
			Container( const Key& key );
			Container( const Container& container );
			virtual ~Container();

			Container& operator=( const Container& container );

			const Key& key() const;
			int year() const;

			BulkPurchaseMap& bulkPurchases();
//			BulkPurchase& purchase();

			void setYear( int nYear );

			static Container fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Container create( const Container& container, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

		protected:
			void loadBulkPurchases( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export ContainerMap : public std::map<Container::Key, Container>
	{
		public:
			static ContainerMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_Container_h__ */
