#ifndef CUSTOMER_H
#define CUSTOMER_H

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Customer : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strCustomer;

				public:
					Key();
					Key( const std::string& strCustomer );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& customer() const;
			};

		private:
			Key _key;
			std::string _strName;
			std::string _strDName;
			std::string _strAName;
			std::string _strOrganisation;
			std::string _strAddress;
			std::string _strTown;
			std::string _strRegion;
			std::string _strCountry;
			std::string _strPostcode;
			std::string _strTelephone;
			std::string _strEmail;

		public:
			Customer();
			Customer( const Key& key );
			Customer( const Customer& customer );
			virtual ~Customer();

			Customer& operator=( const Customer& customer );

			const Key& key() const;
			const std::string& name() const;
			const std::string& dname() const;
			const std::string& aname() const;
			const std::string& organisation() const;
			const std::string& address() const;
			const std::string& town() const;
			const std::string& region() const;
			const std::string& country() const;
			const std::string& postcode() const;
			const std::string& telephone() const;
			const std::string& email() const;

			void setName( const std::string& strName );
			void setDName( const std::string& strDName );
			void setAName( const std::string& strAName );
			void setOrganisation( const std::string& strOrganisation );
			void setAddress( const std::string& strAddress );
			void setTown( const std::string& strTown );
			void setRegion( const std::string& strRegion );
			void setCountry( const std::string& strCountry );
			void setPostcode( const std::string& strPostcode );
			void setTelephone( const std::string& strTelephone );
			void setEmail( const std::string& strEmail );

			static Customer fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Customer create( Customer& customer, RfcDal::DalDatabase& db );
			virtual void update( RfcDal::DalDatabase& db );
	};

	class TaylorsData_Export CustomerMap : public std::map<Customer::Key, Customer>
	{
		public:
			static CustomerMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif // CUSTOMER_H
