#ifndef __TaylorsData_DBProfile_h__
#define __TaylorsData_DBProfile_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export DBProfile : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strVarName;

			public:
				Key();
				Key( const std::string& strVarName );
				Key( const Key& key );
				virtual ~Key();

				const Key& operator=( const Key& key );
				bool operator==( const Key& key ) const;
				bool operator!=( const Key& key ) const;
				bool operator<( const Key& key ) const;

				const std::string& varName() const;
		};

		static const Key keyAutoClass;
		static const Key keyDataPath;
		static const Key keyDefaultCurrency;
		static const Key keyDetailDimDisplay;
		static const Key keyDetailDimensions;
		static const Key keyDetailDimStore;
		static const Key keyDetailInventoryNo;
		static const Key keyDetailLocation;
		static const Key keyDetailPrice;
		static const Key keyDetailSDesc;
		static const Key keyDetailStatus;
		static const Key keyDocPath;
		static const Key keyDomain;
		static const Key keyFrom;
		static const Key keyHeadlines;
		static const Key keyImage0;
		static const Key keyImage1;
		static const Key keyImagePath;
		static const Key keyInvoiceFile;
		static const Key keyInvoiceLines;
		static const Key keyOffline;
		static const Key keySearchGlossary;
		static const Key keySpawnGlossary;
		static const Key keySummaryItems;
		static const Key keySummaryLocation;
		static const Key keySummaryPrice;
		static const Key keySummaryStatus;
		static const Key keyTax1Name;
		static const Key keyTax1Rate;
		static const Key keyTax2Name;
		static const Key keyTax2Rate;
		static const Key keyTax3Name;
		static const Key keyTax3Rate;
		static const Key keyTax4Name;
		static const Key keyTax4Rate;
		static const Key keyTicketFile;
		static const Key keyTimestamp;

	private:
		Key _key;
		std::string _strVarValue;

	public:
		DBProfile();
		DBProfile( const Key& key );
		DBProfile( const DBProfile& profile );
		virtual ~DBProfile();

		const DBProfile& operator=( const DBProfile& profile );

		const Key& key() const;
		const std::string& varValue() const;

		void setVarValue( const std::string& strVarValue );

		static DBProfile fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
		static DBProfile create( const DBProfile& profile, RfcDal::DalDatabase& conn );
		virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export DBProfileMap : public std::map<DBProfile::Key, DBProfile>
	{
		public:
			static DBProfileMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_DBProfile_h__ */
