#ifndef __TaylorsData_Exception_h__
#define __TaylorsData_Exception_h__

#include <TaylorsData/TaylorsData.h>

#include <string>
#include <vector>

namespace TaylorsData
{
	static const int TEX_INVALIDEXCEPTION = -1;
	static const int TEX_COMPOUNDEXCEPTION = 1000;
	static const int TEX_DALEXCEPTION = 1010;
	static const int TEX_BASE4CEXCEPTION = 1011;
	static const int TEX_ALIENEXCEPTION = 1012;
	static const int TEX_SQLEXCEPTION = 1013;
	static const int TEX_DALERROR = 1020;
	static const int TEX_APPLICATIONABORTED = 1030;
	static const int TEX_FILEOPENFAILED = 1050;
	static const int TEX_NOCONNECTSTRING = 1055;
	static const int TEX_MANAGERNOTREGISTERED = 1060;
	static const int TEX_MISSINGDATA = 1100;
	static const int TEX_BADARTICLEID = 1110;
	static const int TEX_NOSUCHDBPROFILE = 1200;
	static const int TEX_NOSUCHSITE = 1201;
	static const int TEX_NOSUCHCATEGORY = 1202;
	static const int TEX_NOSUCHLOCATION = 1203;
	static const int TEX_NOSUCHSUBLOCATION = 1204;
	static const int TEX_NOSUCHSTATUS = 1205;
	static const int TEX_NOSUCHARTICLE = 1206;
	static const int TEX_NOSUCHMETAITEM = 1207;
	static const int TEX_NOSUCHCONTAINER = 1208;
	static const int TEX_NOSUCHBULKPURCHASETYPE = 1209;
	static const int TEX_NOSUCHBULKPURCHASE = 1210;
	static const int TEX_NOSUCHNEWS = 1211;
	static const int TEX_NOSUCHREPORT = 1212;
	static const int TEX_NOSUCHCUSTOMER = 1213;
	static const int TEX_NOSUCHFORMAT = 1214;
	static const int TEX_NODBPROFILESET = 1300;
	static const int TEX_NOSITESET = 1301;
	static const int TEX_NOCATEGORYSET = 1302;
	static const int TEX_NOLOCATIONSET = 1303;
	static const int TEX_NOSUBLOCATIONSET = 1304;
	static const int TEX_NOSTATUSSET = 1305;
	static const int TEX_NOARTICLESET = 1306;
	static const int TEX_NOMETAITEMSET = 1307;
	static const int TEX_NOCONTAINERSET = 1308;
	static const int TEX_NOBULKPURCHASETYPESET = 1309;
	static const int TEX_NOBULKPURCHASESET = 1310;
	static const int TEX_NONEWSSET = 1311;
	static const int TEX_NOREPORTSET = 1312;
	static const int TEX_NOCUSTOMERSET = 1313;
	static const int TEX_NOFORMATSET = 1314;
	static const int TEX_ARTICLEINVALID = 1400;
	static const int TEX_BADFORMATSTRING = 1500;
	static const int TEX_BADUNITSTRING = 1501;
	static const int TEX_REPORTSYNTAXERROR = 1510;

	class TaylorsData_Export Exception
	{
		public:
			static const int TEX_INVALIDLINE;

		private:
			std::string _strFile;
			int _nLine;
			int _nErrorCode;
			std::string _strLegend;

		protected:
			Exception(); // results in the creation of an incomplete Exception

		public:
			Exception( const std::string& strFile, int nLine, int nErrorCode, const std::string& strLegend = std::string() );
			Exception( const Exception& e );
			virtual ~Exception();

			const Exception& operator=( const Exception& e );

			const std::string& file() const;
			int line() const;
			int errorCode() const;
			const std::string& legend() const;

			static std::string explainErrorCode( int nErrorCode );
	};

	class TaylorsData_Export ExceptionList : public std::vector<Exception>
	{
	};

	class TaylorsData_Export CompoundException : public Exception
	{
		private:
			ExceptionList _exceptionS;

		public:
			CompoundException();
			CompoundException( const std::string& strFile, int nLine );
			virtual ~CompoundException();

			const CompoundException& operator=( const CompoundException& e );

			bool hasExceptions() const;
			const ExceptionList& exceptions() const;
			void addException( const Exception& e );
			void addExceptions( const ExceptionList& e );
	};
};

#endif /* !__TaylorsData_Exception_h__ */
