#ifndef __TaylorsData_GlossaryEntry_h__
#define __TaylorsData_GlossaryEntry_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export GlossaryEntry : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strId;

			public:
				Key();
				Key( const std::string& strId );
				Key( const Key& key );
				virtual ~Key();

				const Key& operator=( const Key& key );
				bool operator==( const Key& key ) const;
				bool operator!=( const Key& key ) const;
				bool operator<( const Key& key ) const;

				const std::string& id() const;
			};

		private:
			Key _key;
			std::string _strRedirect;
			std::string _strReplacement;
			std::string _strDescription;
			std::string _strPhoto0;
			int _nDimX0;
			int _nDimY0;

		public:
			GlossaryEntry();
			GlossaryEntry( const Key& key );
			GlossaryEntry( const GlossaryEntry& entry );
			virtual ~GlossaryEntry();

			const GlossaryEntry& operator=( const GlossaryEntry& entry );

			const Key& key() const;
			const std::string& redirect() const;
			const std::string& replacement() const;
			const std::string& description() const;
			const std::string& photo0() const;
			int dimX0() const;
			int dimY0() const;

			void setRedirect( const std::string& strRedirect );
			void setReplacement( const std::string& strReplacement );
			void setDescription( const std::string& strDescription );
			void setPhoto0( const std::string& strPhoto0 );
			void setDimX0( int nDimX0 );
			void setDimY0( int nDimY0 );

			static GlossaryEntry fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static GlossaryEntry create( const GlossaryEntry& entry, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class GlossaryEntryMap : public std::map<GlossaryEntry::Key, GlossaryEntry>
	{
		public:
			static GlossaryEntryMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};

	class GlossaryEntryPtrMap : public std::map<GlossaryEntry::Key, GlossaryEntry*>
	{
	};
}

#endif /* !__TaylorsData_GlossaryEntry_h__ */
