#ifndef __TaylorsData_Image_h__
#define __TaylorsData_Image_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Image : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					int _nArticleId;
					int _nSeq;

				public:
					Key();
					Key( int nArticleId, int nSeq );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int articleId() const;
					int seq() const;
			};

			static const Key keyInvalid;

		private:
			Key _key;
			std::string _strPhoto0; //Thumbnail
			int _nDimX0;
			int _nDimY0;
			std::string _strPhoto1; // Statndard
			int _nDimX1;
			int _nDimY1;
			std::string _strPhoto2; // Large
			int _nDimX2;
			int _nDimY2;

		public:
			Image();
			Image( const Key& key );
			Image( const Image& image );
			virtual ~Image();

			const Image& operator=( const Image& image );

			const Key& key() const;
			const std::string& photo0() const;
			int dimX0() const;
			int dimY0() const;
			const std::string& photo1() const;
			int dimX1() const;
			int dimY1() const;
			const std::string& photo2() const;
			int dimX2() const;
			int dimY2() const;

			void setPhoto0( const std::string& strPhoto0 );
			void setDimX0( int nDimX0 );
			void setDimY0( int nDimY0 );
			void setPhoto1( const std::string& strPhoto1 );
			void setDimX1( int nDimX1 );
			void setDimY1( int nDimY1 );
			void setPhoto2( const std::string& strPhoto2 );
			void setDimX2( int nDimX2 );
			void setDimY2( int nDimY2 );

			static Image fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
//			static Image create( const Image& image, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export ImageMap : public std::map<Image::Key, Image>
	{
		public:
			static ImageMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_Image_h__ */
