#ifndef __TaylorsData_Location_h__
#define __TaylorsData_Location_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class SubLocation;
	class SubLocationMap;

	class TaylorsData_Export Location : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strLocation;

				public:
					Key();
					Key( const std::string& strLocation );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& location() const;
			};

			static const Key keyDefault;

		private:
			Key _key;
			std::string _strDescription;
			bool _bOffline;
			std::string _strURI;
			SubLocationMap* _psubLocationS;

		public:
			Location();
			Location( const Key& key );
			Location( const Location& location );
			virtual ~Location();

			const Location& operator=( const Location& location );

			const Key& key() const;
			const std::string& description() const;
			bool isOffline() const;
			const std::string& uri() const;
			const SubLocationMap& subLocations() const;

			void setDescription( const std::string& strDescription );
			void setOffline( bool bOffline );
			void setURI( const std::string& strURI );
			SubLocationMap& subLocations();

			static Location fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Location create( const Location& location, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

			std::string format() const;
	};

	class TaylorsData_Export LocationMap : public std::map<Location::Key, Location>
	{
		public:
			static LocationMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
};

#endif /* !__TaylorsData_Location_h__ */
