#ifndef __TaylorsData_MetaItem_h__
#define __TaylorsData_MetaItem_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <Base4c/Util.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class MetaItemPtrMap;

	class TaylorsData_Export MetaItem : public Base
	{
		public:
			enum DataType{ dtNone, dtInt, dtReal, dtString, dtDate, dtList };

			class TaylorsData_Export Key
			{
				private:
					std::string _strMetaItem;

				public:
					Key();
					Key( const std::string& strMetaItem );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& metaItem() const;
			};

		private:
			Key _key;
			std::string _strParent;
			DataType _dataType;
			std::string _strDescription;
			std::string _strDefaultValue;
			MetaItemPtrMap* _pchildren;

		public:
			MetaItem();
			MetaItem( const Key& key );
			MetaItem( const MetaItem& item );
			virtual ~MetaItem();

			const MetaItem& operator=( const MetaItem& item );

			const Key& key() const;
			bool hasParent() const;
			const MetaItem& parent() const;
			DataType dataType() const;
			const std::string& description() const;
			const std::string& defaultValue() const;
			const Base4c::StringList defaultValues() const;
			const MetaItemPtrMap& children() const;

			MetaItem& parent();
			void setParent( const MetaItem& item );
			void setDataType( DataType type );
			void setDescription( const std::string& strDescription );
			void setDefaultValue( const std::string& strDefaultValue );
			MetaItemPtrMap& children();

			static MetaItem fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static MetaItem create( const MetaItem& item, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

			static DataType dataTypeFromText( const std::string& strDataType );
			static std::string dataTypeToText( DataType type );
	};

	class TaylorsData_Export MetaItemMap : public std::map<MetaItem::Key, MetaItem>
	{
		public:
			static MetaItemMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};

	class TaylorsData_Export MetaItemPtrMap : public std::map<MetaItem::Key, MetaItem*>
	{
	};
}

#endif // __TaylorsData_MetaItem_h__
