#ifndef __TaylorsData_News_h__
#define __TaylorsData_News_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <Base4c/Guid.h>

namespace TaylorsData
{
	class TaylorsData_Export News : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					Base4c::Guid _guid;

				public:
					Key();
					Key( const Base4c::Guid& guid );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );

					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Guid& news() const;
			};

			static const Key keyInvalid;

		private:
			Key _key;
			int _nSeq;
			std::string _strHeadline;
			std::string _strImage;
			std::string _strImageAlt;
			std::string _strArticleText;

		public:
			News();
			News( const Key& key );
			News( const News& news );
			virtual ~News();

			const News& operator=( const News& news );

			const Key& key() const;
			int seq() const;
			const std::string& headline() const;
			const std::string& image() const;
			const std::string& imageAlt() const;
			const std::string& articleText() const;

			void setSeq( int nSeq );
			void setHeadline( const std::string& strHeadline );
			void setImage( const std::string& strImage );
			void setImageAlt( const std::string& strImageAlt );
			void setArticleText( const std::string& strArticleText );

			static News fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static News create( const News& news, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export NewsMap : public std::map<News::Key, News>
	{
		public:
			static NewsMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_News_h__ */
