#ifndef __TaylorsData_Note_h__
#define __TaylorsData_Note_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Note : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strNoteId;

				public:
					Key();
					Key( const std::string& strNoteId );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& noteId() const;
			};

		private:
			Key _key;
			std::string _strNote;

		public:
			Note();
			Note( const Key& key );
			Note( const Note& note );
			virtual ~Note();

			const Note& operator=( const Note& note );

			const Key& key() const;
			const std::string& note() const;

			void setNote( const std::string& strNote );

			static Note fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Note create( const Note& note, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export NoteMap : public std::map<Note::Key, Note>
	{
		public:
			static NoteMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_Note_h__ */
