#ifndef __TaylorsData_Report_h__
#define __TaylorsData_Report_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/SqlValue.h>
#include <librfcdal/DalDatabase.h>
#include <librfcdal/SqlCondition.h>
#include <librfcdal/SqlConditionPtr.h>
#include <librfcdal/SqlColumnPtr.h>

#include <string>
#include <map>

namespace Rfc
{
	class XMLNode;
}

namespace TaylorsData
{
	class TaylorsData_Export Report : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strReport;

				public:
					Key();
					Key( const std::string& strReport );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );

					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& report() const;
			};

		private:
			Key _key;
			std::string _strDescription;
			RfcDal::SqlValueList _valStatusS;
			std::string _strFilterTemplate;

		public:
			Report();
			Report( const Key& key );
			Report( const Report& report );
			virtual ~Report();

			Report& operator=( const Report& report );

			const Key& key() const;
			const std::string& description() const;
			const RfcDal::SqlValueList& statuss() const;
			const std::string& filterTemplate() const;

			void addStatus( const RfcDal::SqlValue& status );

			static Report fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Report create( const Report& report, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

			RfcDal::SqlConditionPtr filter() const;

		protected:
			static RfcDal::SqlColumnPtr entity( const Rfc::XMLNode* pnode );
			static RfcDal::SqlColumn column( const Rfc::XMLNode* pnode );
			static RfcDal::SqlFunction function( const Rfc::XMLNode* pnode );
			static RfcDal::SqlValue value( const Rfc::XMLNode* pnode );
			static RfcDal::SqlExpression expression( const Rfc::XMLNode* pnode );
			static RfcDal::SqlConditionPtr condition( const Rfc::XMLNode* pnode );
			static RfcDal::SqlCondition conditionCondition( const Rfc::XMLNode* pnode );
			static RfcDal::SqlCompoundCondition conditionCompound( const Rfc::XMLNode* pnode );
			static RfcDal::SqlCondition::Condition condition( const std::string& strCondition );
	};

	class TaylorsData_Export ReportMap : public std::map<Report::Key, Report>
	{
		public:
			static ReportMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
}

#endif /* !__TaylorsData_Report_h__ */
