#ifndef __TaylorsData_Status_h__
#define __TaylorsData_Status_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Status : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					int _nStatus;

				public:
					Key();
					Key( int nStatus );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int status() const;
			};

			static const Key keyDefault;

		private:
			Key _key;
			std::string _strDescription;
			bool _bVisible;
			std::string _strDecoration;
			const Status* _pstatusAdvance;
			int _nDuration;

		public:
			Status();
			Status( const Key& key );
			Status( const Status& status );
			virtual ~Status();

			const Status& operator=( const Status& status );

			const Key& key() const;
			const std::string& description() const;
			bool isVisible() const;
			const std::string& decoration() const;
			bool isAutoAdvance() const;
			const Status& advanceStatus() const;
			int duration() const;

			void setDescription( const std::string& strDescription );
			void setVisible( bool bVisible );
			void setDecoration( const std::string& strDecoration );
			void clearAdvanceStatus();
			void setAdvanceStatus( const Status& status );
			void setDuration( int nDays );

			static Status fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db, bool bGetAdvanceStatus = true );
			void getAdvanceStatus( const RfcDal::DalDatabase::Row& row );
			static Status create( const Status& status, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

			std::string format() const;
	};

	class TaylorsData_Export StatusMap : public std::map<Status::Key, Status>
	{
		public:
			static StatusMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
			void getAdvanceStatus( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
};

#endif /* !__TaylorsData_Status_h__ */
