#ifndef __TaylorsData_SubLocation_h__
#define __TaylorsData_SubLocation_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>
#include <TaylorsData/Location.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export SubLocation : public Base
	{
		public:
			class TaylorsData_Export Key : public Location::Key
			{
				private:
					std::string _strSubLocation;

				public:
					Key();
					Key( const Location::Key& keyLocation, const std::string& strSubLocation );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& subLocation() const;
			};

		private:
			Key _key;
			Location* _plocation;
			std::string _strDescription;

		public:
			SubLocation();
			SubLocation( const Key& key );
			SubLocation( const SubLocation& location );
			virtual ~SubLocation();

			const SubLocation& operator=( const SubLocation& location );

			const Key& key() const;
			Location& location() const;
			const std::string& description() const;

			void setLocation( Location& location );
			void setDescription( const std::string& strDescription );

			static SubLocation& fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static SubLocation& create( const SubLocation& location, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );

			std::string format() const;
	};

	class TaylorsData_Export SubLocationMap : public std::map<SubLocation::Key, SubLocation>
	{
		public:
			static void fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
};

#endif /* !__TaylorsData_SubLocation_h__ */
