#ifndef __TaylorsData_TaylorsFactory_h__
#define __TaylorsData_TaylorsFactory_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/DBProfile.h>
#include <TaylorsData/Site.h>
#include <TaylorsData/Category.h>
#include <TaylorsData/Location.h>
#include <TaylorsData/Status.h>
#include <TaylorsData/Note.h>
#include <TaylorsData/MetaItem.h>
#include <TaylorsData/Article.h>
#include <TaylorsData/Glossary.h>
#include <TaylorsData/News.h>
#include <TaylorsData/Container.h>
#include <TaylorsData/BulkPurchaseType.h>
#include <TaylorsData/Report.h>
#include <TaylorsData/Format.h>
#include <TaylorsData/Customer.h>

#include <Base4c/Version.h>

#include <librfcdal/SqlConditionPtr.h>

#include <string>

namespace TaylorsData
{
	class TaylorsData_Export TaylorsFactory
	{
		private:
			static DBProfileMap _dbProfileS;
			static SiteMap _siteS;
			static CategoryMap _categoryS;
			static CategoryPtrMap _categoryRootS;
			static LocationMap _locationS;
			static StatusMap _statusS;
			static NoteMap _noteS;
			static MetaItemMap _metaItemS;
			static MetaItemPtrMap _metaRootS;
			static ArticleMap _articleS;
			static Glossary _glossary;
			static NewsMap _newsS;
			static ContainerMap _containerS;
			static BulkPurchaseTypeMap _bulkPurchaseTypeS;
			static ReportMap _reportS;
			static FormatMap _formatS;
			static CustomerMap _customerS;

			static Article _articleDefault;

			static bool _bAIPIntegrated;

		public:
			static bool isAIPIntegrated();
			static void setAIPIntegrated( bool bIntegrated );

			static void initialize( const std::string& strInit );

			static DBProfileMap& dbProfiles();
			static SiteMap& sites();
			static CategoryMap& categorys();
			static CategoryPtrMap& categoryRoots();
			static LocationMap& locations();
			static StatusMap& statuss();
			static NoteMap& notes();
			static MetaItemMap& metaItems();
			static MetaItemPtrMap& metaItemRoots();
			static ArticleMap& articles();
			static NewsMap& news();
			static ContainerMap& containers();
			static BulkPurchaseTypeMap& bulkPurchaseTypes();
			static ReportMap& reports();
			static FormatMap& formats();
			static CustomerMap& customers();

			static Glossary& glossary();

			static const DBProfile& dbProfile( const DBProfile::Key& key );

			static Article& article( const Article::Key& key );

			static const Base4c::Version version();

			static Article::KeySet getArticleIds( RfcDal::DalDatabase& db, const RfcDal::SqlConditionPtr& cond );
			static Article::KeySet getArticleIds( RfcDal::DalDatabase& db, const Location& location );
			static Article::KeySet getArticleIds( RfcDal::DalDatabase& db, const SubLocation& location );
			static ArticlePtrMap getArticles( RfcDal::DalDatabase& db, const Article::KeySet& keyS );

			static void exportArticleToCADA( Article& article );

		protected:
			static void loadDBProfiles( RfcDal::DalDatabase& db );
			static void loadSites( RfcDal::DalDatabase& db );
			static void loadCategorys( RfcDal::DalDatabase& db );
			static void loadLocations( RfcDal::DalDatabase& db );
			static void loadStatuss( RfcDal::DalDatabase& db );
			static void loadNotes( RfcDal::DalDatabase& db );
			static void loadMetaItems( RfcDal::DalDatabase& db );
			static void loadNews( RfcDal::DalDatabase& db );
			static void loadContainers( RfcDal::DalDatabase& db );
			static void loadBulkPurchaseTypes( RfcDal::DalDatabase& db );
			static void loadReports( RfcDal::DalDatabase& db );
			static void loadFormats( RfcDal::DalDatabase& db );
			static void loadCustomers( RfcDal::DalDatabase& db );
			static void loadDefaultArticle();
	};
};

#endif /* !__TaylorsData_TaylorsFactory_h__ */
