#ifndef ARTICLETABLEWIDGETITEM_H
#define ARTICLETABLEWIDGETITEM_H

#include <TaylorsData/Article.h>
#include <TaylorsData/BulkPurchase.h>

#include <Base4c/Subscriber.h>

#include <QTableWidgetItem>

namespace TaylorsUIComponents
{
	class ArticleTableWidgetItem : public QTableWidgetItem, public Base4c::Subscriber
	{
		private:
			TaylorsData::Article* _particle;
			TaylorsData::BulkPurchase* _ppurchase;
			bool _bDirty;
			std::string _strMethod;

		public:
			ArticleTableWidgetItem( TaylorsData::Article* article, TaylorsData::BulkPurchase* ppurchase );
			virtual ~ArticleTableWidgetItem();

			TaylorsData::Article& article();
			TaylorsData::BulkPurchase& purchase();
			bool isDirty() const;
			const std::string& method() const;

			void setMethod( const std::string& strMethod );

			virtual void setDirty();
			virtual void save();
			virtual void doubleClicked();
			virtual void refresh()= 0;
			virtual void evaluate();

			virtual void notify( Base4c::Publisher* ppublisher, Base4c::Publisher::Notification notification, void* pnotifier );
	};

	class ArticleIdTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			ArticleIdTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~ArticleIdTableWidgetItem();

			virtual void save();
			virtual void refresh();
	};

	class InventoryNoTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			InventoryNoTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~InventoryNoTableWidgetItem();

			virtual void save();
			virtual void refresh();
	};

	class CategoryTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			CategoryTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~CategoryTableWidgetItem();

			virtual void save();
			virtual void doubleClicked();
			virtual void refresh();
	};

	class ShortDescTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			ShortDescTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~ShortDescTableWidgetItem();

			virtual void save();
			virtual void refresh();
	};

	class DescriptionTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			DescriptionTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~DescriptionTableWidgetItem();

			virtual void save();
			virtual void refresh();
	};

	class CircaTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			CircaTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~CircaTableWidgetItem();

			virtual void save();
			virtual void refresh();
	};

	class PurchasePriceTableWidgetItem : public ArticleTableWidgetItem
	{
		public:
			PurchasePriceTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase );
			virtual ~PurchasePriceTableWidgetItem();

			virtual void save();
			virtual void refresh();
			virtual void evaluate();
	};

	class ArticleMetaTableWidgetItem : public ArticleTableWidgetItem
	{
		private:
			const TaylorsData::MetaItem* _pmetaItem;
			TaylorsData::ArticleMetaItem* _particleMetaItem;

		public:
			ArticleMetaTableWidgetItem( TaylorsData::Article* particle, TaylorsData::BulkPurchase* ppurchase, const TaylorsData::MetaItem* pmetaItem );
			virtual ~ArticleMetaTableWidgetItem();

			TaylorsData::ArticleMetaItem* articleMetaItem();

			virtual void save();
			virtual void refresh();
			virtual void evaluate();
	};
}

#endif // ARTICLETABLEWIDGETITEM_H
