#ifndef UICOMPONENTMANAGER_H
#define UICOMPONENTMANAGER_H

#include <TaylorsUIComponents/ArticleTreeWidgetItem.h>
#include <TaylorsUIComponents/ArticleWindow.h>
#include <TaylorsUIComponents/NewsWindow.h>
#include <TaylorsUIComponents/BulkPurchaseWindow.h>
#include <TaylorsUIComponents/AntiqueManagerChildWindow.h>

#include <TaylorsData/Article.h>

#include <set>
#include <map>

namespace TaylorsUIComponents
{
	class UIComponentManager
	{
		public:
			class ArticleWindowMap : public std::map<TaylorsData::Article::Key, ArticleWindow*>
			{
			};

			class NewsWindowMap : public std::map<TaylorsData::News::Key, NewsWindow*>
			{
			};

			class BulkPurchaseWindowMap : public std::map<TaylorsData::BulkPurchase::Key, BulkPurchaseWindow*>
			{
			};

			class ChildWindowSet : public std::set<AntiqueManagerChildWindow*>
			{
			};

		private:
			static UIComponentManager* _pmanager;
			ArticleWindowMap _windowSArticle;
			NewsWindowMap _windowSNews;
			BulkPurchaseWindowMap _windowSBulkPurchase;
			ChildWindowSet _children;

		public:
			UIComponentManager();
			virtual ~UIComponentManager();

			ArticleWindow* articleWindow( const TaylorsData::Article& article );
			ArticleWindow* articleWindow( const TaylorsData::Article::Key& key );
			NewsWindow* newsWindow( const TaylorsData::News& news );
			NewsWindow* newsWindow( const TaylorsData::News::Key& key );
			BulkPurchaseWindow* bulkPurchaseWindow( const TaylorsData::BulkPurchase& purchase );
			BulkPurchaseWindow* bulkPurchaseWindow( const TaylorsData::BulkPurchase::Key& key );
			ChildWindowSet& amChildren();


			void registerArticleWindow( ArticleWindow* pwindow );
			void unregisterArticleWindow( ArticleWindow* pwindow );

			void registerNewsWindow( NewsWindow* pwindow );
			void unregisterNewsWindow( NewsWindow* pwindow );

			void registerBulkPurchaseWindow( BulkPurchaseWindow* pwindow );
			void unregisterBulkPurchaseWindow( BulkPurchaseWindow* pwindow );

			virtual void registerArticle( TaylorsData::Article& article ) = 0;

			static UIComponentManager& manager();

		protected:
			void registerChildWindow( AntiqueManagerChildWindow* pwindow );
			void unregisterChildWindow( AntiqueManagerChildWindow* pwindow );

			void registerUIComponentManager( UIComponentManager* pmanager );
			void unregisterUIComponentManager();
	};
}

#endif // UICOMPONENTMANAGER_H
